var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { getMultiCallContract, getOffchainOracleContract } from './contracts/factories';
import { ethers, BigNumber } from 'ethers';
import { defaultAbiCoder } from 'ethers/lib/utils';
import { instances } from '@tornado/tornado-config';
import { ChainId } from './config';
/**
 * Filter non-native tokens from Tornado instances config
 * @param instances Tornado instances from torn-token config ('@tornado/tornado-config' library)
 * @returns Array of non-native tokens
 */
function filterTokensFromTornadoInstances(instances) {
    const ethInstances = Object.values(instances[ChainId.MAINNET]);
    return ethInstances.filter((tokenInstance) => !!tokenInstance.tokenAddress);
}
const tornToken = {
    tokenAddress: '0x77777FeDdddFfC19Ff86DB637967013e6C6A116C',
    symbol: 'torn',
    decimals: 18,
};
const tornadoTokens = [tornToken, ...filterTokensFromTornadoInstances(instances)];
const defaultTornadoTokenPrices = {
    torn: '1689423546359032',
    dai: '598416104472725',
    cdai: '13384388487019',
    usdc: '599013776676721',
    usdt: '599323410893614',
    wbtc: '15659889148334216720',
};
export class TokenPriceOracle {
    /**
     *
     * @param rpcUrl
     * @param tokens
     */
    constructor(rpcUrl, tokens = tornadoTokens, defaultTokenPrices = defaultTornadoTokenPrices) {
        this.tokens = tokens;
        this.defaultTokenPrices = defaultTokenPrices;
        this.provider = new ethers.providers.JsonRpcProvider(rpcUrl);
        this.oracle = getOffchainOracleContract(this.provider);
        this.multiCall = getMultiCallContract(this.provider);
    }
    // Instant return default token prices
    get defaultPrices() {
        return this.defaultTokenPrices;
    }
    /**
     * Prepare data for MultiCall contract
     * @param {Token[]} tokens Tokens array
     * @returns Valid structure to provide to MultiCall contract
     */
    prepareCallData(tokens = this.tokens) {
        return tokens.map((token) => ({
            to: this.oracle.address,
            data: this.oracle.interface.encodeFunctionData('getRateToEth', [token.tokenAddress, true]),
        }));
    }
    /**
     * Fetch actual tokens price rate to ETH from offchain oracles
     * @param {Token[]} tokens Token array
     * @returns {TokenPrices} Object with token price rate to ETH in WEI
     */
    fetchPrices(tokens = this.tokens) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!(tokens === null || tokens === void 0 ? void 0 : tokens.length))
                    return {};
                const callData = this.prepareCallData(tokens);
                const { results, success } = yield this.multiCall.multicall(callData);
                const prices = {};
                for (let i = 0; i < results.length; i++) {
                    const tokenSymbol = tokens[i].symbol.toLowerCase();
                    if (!success[i]) {
                        if (this.defaultTokenPrices[tokenSymbol])
                            prices[tokenSymbol] = this.defaultTokenPrices[tokenSymbol];
                        continue;
                    }
                    const decodedRate = defaultAbiCoder.decode(['uint256'], results[i]).toString();
                    const tokenDecimals = BigNumber.from(10).pow(tokens[i].decimals);
                    const ethDecimals = BigNumber.from(10).pow(18);
                    const price = BigNumber.from(decodedRate).mul(tokenDecimals).div(ethDecimals);
                    prices[tokenSymbol] = price.toString();
                }
                return prices;
            }
            catch (e) {
                console.error('Cannot get token prices, return default: ' + e);
                return this.defaultTokenPrices;
            }
        });
    }
}
//# sourceMappingURL=tokenPriceOracle.js.map