import { GasPriceOracle } from '@tornado/gas-price-oracle';
import { BigNumberish } from 'ethers';
import { TransactionData, TxType, ITornadoFeeOracle, LegacyGasPriceKey, GasPriceParams, GetGasParamsRes, HexadecimalStringifiedNumber, GetGasInput, GetGasParamsInput } from './types';
import { JsonRpcProvider } from '@ethersproject/providers';
import { ChainId, InstanceTokenSymbol } from './config';
import { GetWithdrawalFeeViaRelayerInput } from './types';
export declare abstract class TornadoFeeOracle implements ITornadoFeeOracle {
    version: 4 | 5;
    protected chainId: ChainId;
    protected oracle: GasPriceOracle;
    protected provider: JsonRpcProvider;
    constructor(version: 4 | 5, chainId: ChainId, rpcUrl: string, oracle: GasPriceOracle);
    /**
     * Because Optimism transaction published on Mainnet, for each OP transaction we need to calculate L1 security fee:
     * https://community.optimism.io/docs/developers/build/transaction-fees/#priority-fee
     * @param {TransactionData} tx Transaction data to estimate L1 additional fee
     * @returns {Promise<HexadecimalStringifiedNumber>} Fee in WEI (MATIC), '0' if chain is not Optimism
     */
    fetchL1OptimismFee(tx?: TransactionData): Promise<HexadecimalStringifiedNumber>;
    /**
     * Estimate gas price, gas limit and l1Fee for sidechain (if exists)
     * @param {GetGasParamsInput} params Function input arguments object
     * @param {TransactionData} [params.tx] Transaction data in web3 / ethers format
     * @param {TxType} [params.txType=other] Tornado transaction type: withdrawal by user, withdrawal by relayer or 'other'
     * @param {number} [params.predefinedGasLimit] Predefined gas limit, if already calculated (no refetching)
     * @param {number} [params.predefinedGasPrice] Predefined gas price, if already calculated (no refetching)
     * @param {number} [params.bumpGasLimitPercent] Gas limit bump percent to prioritize transaction (if gas limit not predefined, recenlty used)
     * @param {number} [params.bumpGasPricePercent] Gas price bump percent to prioritize transaction (if gas limit not predefined, rarely used)
     * @param {LegacyGasPriceKey} [params.speed] Preferred transaction speed, if uses legacy gas (before EIP-1559)
     * @param {boolean} [params.includeL1FeeToGasLimit=true] Include L1 additional fee on Optimism to gas limit (get fee and divide by gas price)
     * @returns {Promise<GetGasParamsRes>} Object with fields 'gasPrice' and 'gasLimit', L1 fee, if exists, included in gasLimit
     */
    getGasParams(params?: GetGasParamsInput): Promise<GetGasParamsRes>;
    /**
     * Estimates next block gas for signed, unsigned or incomplete Tornado transaction
     * @param {GetGasInput} params Function input arguments object
     * @param {TransactionData} [params.tx] Transaction data in web3 / ethers format
     * @param {TxType} [params.txType] Tornado transaction type: withdrawal by user, withdrawal by relayer or 'other'
     * @param {number} [params.predefinedGasLimit] Predefined gas limit, if already calculated (no refetching)
     * @param {number} [params.predefinedGasPrice] Predefined gas price, if already calculated (no refetching)
     * @param {number} [params.bumpGasLimitPercent] Gas limit bump percent to prioritize transaction (if gas limit not predefined, recenlty used)
     * @param {number} [params.bumpGasPricePercent] Gas price bump percent to prioritize transaction (if gas price not predefined, rarely used)
     * @param {LegacyGasPriceKey} [params.speed] Preferred transaction speed, if uses legacy gas (before EIP-1559)
     * @returns {Promise<HexadecimalStringifiedNumber>} Gas value in WEI (hex-format)
     */
    getGas(params?: GetGasInput): Promise<HexadecimalStringifiedNumber>;
    /**
     * Estimate next block gas price
     * @param {LegacyGasPriceKey} [speed] Preferred transaction speed, if uses legacy gas (before EIP-1559)
     * @param {number} [bumpPercent=0] Gas bump percent to prioritize transaction
     * @returns {Promise<GasPriceParams>} Estimated gas price info in WEI (hexed) - legacy object with gasPrice property or
     * EIP-1559 object with maxFeePerGas and maxPriorityFeePerGas properties
     */
    getGasPriceParams(speed?: LegacyGasPriceKey, bumpPercent?: number): Promise<GasPriceParams>;
    /**
     * Estimate next block gas price
     * @param {LegacyGasPriceKey} speed Preferred transaction speed, if uses legacy gas (before EIP-1559)
     * @param {number} bumpPercent Gas bump percent to prioritize transaction
     * @returns {Promise<HexadecimalStringifiedNumber>} Gas price in WEI (hex string)
     */
    getGasPrice(speed?: LegacyGasPriceKey, bumpPercent?: number): Promise<HexadecimalStringifiedNumber>;
    /**
     * Estimates gas limit for transaction (or basic gas limit, if no tx data provided)
     * @param {TransactionData} tx Transaction data (object in web3 / ethers format)
     * @param {TxType} type Tornado transaction type: withdrawal by user, withdrawal by relayer, relayer fee check or 'other'
     * @param {number} bumpPercent Gas bump percent to prioritize transaction
     * @returns {Promise<number>} Gas limit
     */
    abstract getGasLimit(tx?: TransactionData, type?: TxType, bumpPercent?: number): Promise<number>;
    /**
     * If user withdraw non-native tokens on ETH or Goerli, we need to calculate refund value:
     * if the withdrawal is successful, this amount will be returned to the user after the transfer to the relayer,
     * and if the relayer pays a commission and the transfer of tokens fails, this commission will remain to the relayer.
     *
     * Refund needed that recipient can use tokens after withdrawal (covers gas fee for send/swap)
     * @param {BigNumberish} gasPrice Actual gas price
     * @param {InstanceTokenSymbol} tokenSymbol Withdrawal token (currency) symbol - for example, 'dai'
     * @returns {HexadecimalStringifiedNumber} Refund amount in WEI (in hex format)
     */
    calculateRefundInETH(gasPrice: BigNumberish, tokenSymbol: InstanceTokenSymbol): HexadecimalStringifiedNumber;
    /**
     * Fetched actual gas price and calculates refund amount
     * @param {InstanceTokenSymbol} tokenSymbol Withdrawal token (currency) symbol - for example, 'dai'
     * @returns {Promise<HexadecimalStringifiedNumber>} Refund amount in WEI (in hex format)
     */
    fetchRefundInETH(tokenSymbol: InstanceTokenSymbol): Promise<HexadecimalStringifiedNumber>;
    /**
     * Get refund amount on ETH or Goerli in non-native token
     * @param {BigNumberish} gasPrice Actual gas price in ETH
     * @param {BigNumberish} tokenPriceInEth Token price in WEI in ETH
     * @param {HexadecimalStringifiedNumber | number} tokenDecimals Token (currency) decimals
     * @param {InstanceTokenSymbol} tokenSymbol Withdrawal token (currency) symbol - for example, 'dai'
     * @returns {HexadecimalStringifiedNumber} Refund amount in WEI in selected token (hexed number)
     */
    calculateRefundInToken(gasPrice: BigNumberish, tokenPriceInEth: BigNumberish, tokenDecimals: HexadecimalStringifiedNumber | number, tokenSymbol: InstanceTokenSymbol): HexadecimalStringifiedNumber;
    /**
     * Calculates relayer fee in selected currency (ETH, DAI, BNB etc) in WEI
     * @param {number | string} relayerFeePercent Relayer percent (0.4 for ETH Mainnet, for example)
     * @param {HexadecimalStringifiedNumber | number} amount Amount in selected currency (10 for 10 ETH, 1000 for 1000 DAI)
     * @param {string | number} decimals Decimal places in selected token (currency)
     * @returns {HexadecimalStringifiedNumber} Fee in WEI (hexed stingified number)
     */
    calculateRelayerFeeInWei(relayerFeePercent: number | string, amount: HexadecimalStringifiedNumber | number, decimals: string | number): HexadecimalStringifiedNumber;
    /**
     * Estimates fee for withdrawal via relayer depending on type: gas bump percent is bigger, if it calculates by user,
     * so that the real commission from the relayer side is a little less,
     * in order to the relayer can send a transaction without fear that he will go into the red
     * @param {GetWithdrawalFeeViaRelayerInput} params Function input arguments object
     * @param {TxType} params.type Tornado transaction type: withdrawal costs calculation from user side or from relayer side
     * @param {TransactionData} [params.tx] Transaction data (object in web3 / ethers format)
     * @param {number} params.relayerFeePercent Relayer fee percent from the transaction amount (for example, 0.15 for BNB or 0.4 for ETH Mainnet)
     * @param {AvailableTokenSymbols | Uppercase<AvailableTokenSymbols>} params.currency Currency symbol
     * @param {number | HexadecimalStringifiedNumber } params.amount Withdrawal amount in selected currency
     * @param {number | HexadecimalStringifiedNumber } params.decimals Token (currency) decimals
     * @param {BigNumberish} [params.refundInEth] Refund in ETH, if withdrawed other tokens on Mainnet (not ETH). Can not be provided, if user-side calculation
     * @param {BigNumberish} [params.tokenPriceInEth] If withdrawing other token on Mainnet or Goerli, need to provide token price in ETH (in WEI)
     * @param {number} [params.gasLimit] Predefined gas limit, if already calculated (no refetching)
     * @param {number} [params.gasPrice] Predefined gas price, if already calculated (no refetching)
     *
     * @returns {Promise<HexadecimalStringifiedNumber>} Fee in WEI (hexed string)
     */
    calculateWithdrawalFeeViaRelayer({ tx, txType, relayerFeePercent, currency, amount, decimals, refundInEth, tokenPriceInEth, predefinedGasLimit, predefinedGasPrice, }: GetWithdrawalFeeViaRelayerInput): Promise<HexadecimalStringifiedNumber>;
}
